'use strict';

const fetch = require('node-fetch');
const ms = require('ms');

class RequestHandler {
  constructor(client) {
    this._client = client;
  }
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  async request(endpoint, query = {}, method = "GET", body, _attempts = 0) {
    return new Promise(async (resolve, reject) => {
      const url = `${this._client.baseURL}/${this._client.version}${endpoint}${toQueryString(query)}`;
      const options = {
        method,
        headers: {
          Authorization: this._client.apiKey,
          "Content-Type": "application/json"
        },
        body: body == null || Object.entries(body).length == 0 ? void 0 : JSON.stringify(body),
        timeout: 15e3
      };
      if (this._client.debug)
        console.debug(`Sending request to ${url}
Method:
  ${options.method}
Params:
  ${JSON.stringify(query)}`);
      try {
        const res = this._client.customFetch ? await this._client.customFetch(url, options) : await fetch(url, options);
        if (res.status >= 200 && res.status < 300) {
          const json = await res.json();
          resolve(json);
          if (this._client.debug)
            console.debug("Success: \n", json);
        } else if (res.status === 429) {
          const json = await res.json();
          if (this._client.debug)
            console.debug("Ratelimited: \n", res, json);
          reject(new RatelimitError(res));
        } else {
          try {
            const json = await res.json();
            if (this._client.debug)
              console.debug("API Error: \n", res, json);
            reject(new AmariError(res, json));
          } catch (err) {
            if (this._client.debug)
              console.debug("API Error: \n", res);
            reject(new AmariError(res));
          }
        }
      } catch (err) {
        reject(err);
      }
    });
  }
}
const encode = (str) => {
  const charMap = {
    "!": "%21",
    "'": "%27",
    "(": "%28",
    ")": "%29",
    "~": "%7E",
    "%20": "+",
    "%00": "\0"
  };
  return encodeURIComponent(str).replace(/[!'\(\)~]|%20|%00/g, (match) => {
    return charMap[match];
  });
};
const toQueryString = (data) => {
  if (Object.entries(data).length == 0)
    return "";
  let result = "";
  for (const [key, value] of Object.entries(data)) {
    result.length == 0 ? result = "?" : result += "&";
    result += `${encode(key)}=${encode(value)}`;
  }
  return result;
};

class AmariBot {
  /**
   *
   * @description This is the main class that you initalize to perform all the requests to the API
   * @param {string} apiKey - The token you use to authenticate to the API
   * @param {AmariBotOptions} options - Additional options for the API handler
   */
  constructor(apiKey, options = {}) {
    this.apiKey = apiKey;
    this.debug = options.debug || false;
    this.baseURL = options.baseURL || "https://amaribot.com/api";
    this.version = options.version || "v1";
    if (options.customFetch)
      this.customFetch = options.customFetch;
    this._requestHandler = new RequestHandler(this);
    if (this.debug)
      console.debug("amaribot.js initalized\n" + JSON.stringify(options, null, 2));
  }
  async getUserLevel(guildId, userId) {
    if (this.debug)
      console.debug(`Event: getUserLevel
  - Guild: ${guildId}
  - User: ${userId}`);
    const data = await this._request(`/guild/${guildId}/member/${userId}`);
    return data;
  }
  async getBulkUserLevel(guildId, userId) {
    if (this.debug)
      console.debug(`Event: getBulkUserLevel
  - Guild: ${guildId}
  - Users: ${userId}`);
    const data = await this._request(`/guild/${guildId}/members`, {}, "POST", { members: userId });
    return data;
  }
  async getLeaderboard(guildId, options = {}) {
    if (this.debug)
      console.debug(`Event: getUserLevel
  - Guild: ${guildId}
  - Options: ${JSON.stringify(options, null, 2)}`);
    const data = await this._request(`/guild/leaderboard/${guildId}`, options);
    return data;
  }
  async getRawLeaderboard(guildId, options = {}) {
    if (this.debug)
      console.debug(`Event: getUserLevel
  - Guild: ${guildId}
  - Options: ${JSON.stringify(options, null, 2)}`);
    const data = await this._request(`/guild/raw/leaderboard/${guildId}`, options);
    return data;
  }
  async getWeeklyLeaderboard(guildId, options = {}) {
    if (this.debug)
      console.debug(`Event: getUserLevel
  - Guild: ${guildId}
  - Options: ${JSON.stringify(options, null, 2)}`);
    const data = await this._request(`/guild/weekly/${guildId}`, options);
    return data;
  }
  async getRawWeeklyLeaderboard(guildId, options = {}) {
    if (this.debug)
      console.debug(`Event: getUserLevel
  - Guild: ${guildId}
  - Options: ${JSON.stringify(options, null, 2)}`);
    const data = await this._request(`/guild/raw/weekly/${guildId}`, options);
    return data;
  }
  /**
   * @description Combines the main leaderboard and the weekly leaderboard into one
   */
  async getCombinedLeaderboard(guildId) {
    if (this.debug)
      console.debug(`Event: getUserLevel
  - Guild: ${guildId}`);
    const mainLeaderboard = await this.getRawLeaderboard(guildId);
    const weeklyLeaderboard = await this.getRawWeeklyLeaderboard(guildId);
    const combinedLeaderboard = mainLeaderboard.data.map((x) => {
      const weeklyData = weeklyLeaderboard.data.find((y) => y.id === x.id);
      const user = { ...x, weeklyExp: weeklyData?.exp ?? 0 };
      return user;
    });
    return {
      data: combinedLeaderboard,
      count: combinedLeaderboard.length
    };
  }
  async getGuildRewards(guildId, options = {}) {
    if (this.debug)
      console.debug(`Event: getGuildRewards
  - Guild: ${guildId}
  - Options: ${JSON.stringify(options, null, 2)}`);
    const data = await this._request(`/guild/rewards/${guildId}`, options);
    return data;
  }
  async getLeaderboardPosition(guildId, userId) {
    if (this.debug)
      console.debug(`Event: getLeaderboardPosition
  - Guild: ${guildId}
  - User: ${userId}`);
    const lb = await this.getRawLeaderboard(guildId, { limit: 5e5 });
    const userData = lb.data.find((x) => x.id == userId);
    if (!userData)
      throw new Error(`User ${userId} not found`);
    const position = lb.data.indexOf(userData);
    return position + 1;
  }
  async getWeeklyLeaderboardPosition(guildId, userId) {
    if (this.debug)
      console.debug(`Event: getWeeklyLeaderboardPosition
  - Guild: ${guildId}
  - User: ${userId}`);
    const lb = await this.getRawWeeklyLeaderboard(guildId, { limit: 5e5 });
    const userData = lb.data.find((x) => x.id == userId);
    if (!userData)
      throw new Error(`User ${userId} not found`);
    const position = lb.data.indexOf(userData);
    return position + 1;
  }
  getNextLevelExp(level) {
    if (this.debug)
      console.debug(`Event: getLevelExp
  - Level: ${level}`);
    let value = 20 * (level * level) + 35;
    return value;
  }
  /**
   * @private
   */
  _request(endpoint, query = {}, method = "GET", body = {}) {
    return this._requestHandler.request(endpoint, query, method, body);
  }
}

class AmariError extends Error {
  constructor(response, data) {
    super();
    this.name = this.constructor.name;
    this.status = response.status;
    this.message = data ? data.error : "No message provided";
  }
}

class RatelimitError extends Error {
  constructor(response, data) {
    super();
    this.name = this.constructor.name;
    this.status = response.status;
    this.remaining = parseInt(response.headers.get("Ratelimit-Remaining") ?? `${Date.now()}`);
    this.message = data ? data.error : "You are currently ratelimited! Try again in " + ms(this.remaining);
  }
}

exports.AmariBot = AmariBot;
exports.AmariError = AmariError;
exports.RatelimitError = RatelimitError;
exports.RequestHandler = RequestHandler;
