import { RequestInit, Response as Response$1 } from 'node-fetch';

declare class RequestHandler {
    _client: AmariBot;
    constructor(client: AmariBot);
    request(endpoint: string, query: {} | undefined, method: string | undefined, body: {
        [key: string]: any;
    }, _attempts?: number): Promise<unknown>;
}

declare class AmariBot {
    apiKey: string;
    debug: boolean;
    baseURL: string;
    version: string;
    customFetch?: CustomFetch;
    /**
     * @private
     */
    _requestHandler: RequestHandler;
    /**
     *
     * @description This is the main class that you initalize to perform all the requests to the API
     * @param {string} apiKey - The token you use to authenticate to the API
     * @param {AmariBotOptions} options - Additional options for the API handler
     */
    constructor(apiKey: string, options?: AmariBotOptions);
    getUserLevel(guildId: string, userId: string | string[]): Promise<APIUser>;
    getBulkUserLevel(guildId: string, userId: string[]): Promise<APIBulkUsers>;
    getLeaderboard(guildId: string, options?: PaginationOptions): Promise<APILeaderboard>;
    getRawLeaderboard(guildId: string, options?: RawPaginationOptions): Promise<APIRawLeaderboard>;
    getWeeklyLeaderboard(guildId: string, options?: PaginationOptions): Promise<APIWeeklyLeaderboard>;
    getRawWeeklyLeaderboard(guildId: string, options?: RawPaginationOptions): Promise<APIRawWeeklyLeaderboard>;
    /**
     * @description Combines the main leaderboard and the weekly leaderboard into one
     */
    getCombinedLeaderboard(guildId: string): Promise<{
        data: APIUser[];
        count: number;
    }>;
    getGuildRewards(guildId: string, options?: PaginationOptions): Promise<APIRewards>;
    getLeaderboardPosition(guildId: string, userId: string): Promise<number>;
    getWeeklyLeaderboardPosition(guildId: string, userId: string): Promise<number>;
    getNextLevelExp(level: number): number;
    /**
     * @private
     */
    _request(endpoint: string, query?: {
        [key: string]: string | number;
    }, method?: string, body?: {
        [key: string]: any;
    }): Promise<unknown>;
}

/**
 * @extends Error
 * @property {string} name The type of error (APIError)
 * @property {number} status HTTP status of the error
 * @property {string} message The message of this error
 */

declare class AmariError extends Error {
    name: string;
    status: number;
    message: string;
    constructor(response: CustomResponse, data?: APIError);
}

/**
 * @extends Error
 * @property {string} name The type of error (RatelimitError)
 * @property {number} status HTTP status of the error
 * @property {number} remaining The time remaining for your ratelimit to expire in milliseconds
 * @property {string} message The message of this error
 */
declare class RatelimitError extends Error {
    name: string;
    status: number;
    remaining: number;
    message: string;
    constructor(response: CustomResponse, data?: APIError);
}

type AmariBotOptions = {
    baseURL?: string;
    version?: string;
    debug?: boolean;
    customFetch?: CustomFetch;
};
type CustomFetch = (url: string, options: RequestInit) => Promise<CustomResponse>;
type CustomResponse = Response$1 | Response;
type PaginationOptions = {
    limit?: number;
    page?: number;
};
type RawPaginationOptions = Omit<PaginationOptions, "page">;
type APIError = {
    error: string;
};
type APIUser = {
    id: string;
    username: string;
    exp: number;
    level: number;
    weeklyExp: number;
};
type APILeaderboardUser = Omit<APIUser, "weeklyExp">;
type APIWeeklyUser = Omit<APIUser, "weeklyExp" | "level">;
type APIBulkUsers = {
    members: APIUser[];
    total_members: number;
    queried_members: number;
};
type APILeaderboard = {
    data: APILeaderboardUser[];
    count: number;
    total_count: number;
};
type APIRawLeaderboard = Omit<APILeaderboard, "total_count">;
type APIWeeklyLeaderboard = {
    data: APIWeeklyUser[];
    count: number;
    total_count: number;
};
type APIRawWeeklyLeaderboard = Omit<APIWeeklyLeaderboard, "total_count">;
type APIRewards = {
    data: {
        roleID: string;
        level: number;
    }[];
    count: number;
};

export { APIBulkUsers, APIError, APILeaderboard, APILeaderboardUser, APIRawLeaderboard, APIRawWeeklyLeaderboard, APIRewards, APIUser, APIWeeklyLeaderboard, APIWeeklyUser, AmariBot, AmariBotOptions, AmariError, CustomFetch, CustomResponse, PaginationOptions, RatelimitError, RawPaginationOptions, RequestHandler };
